<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps;

use Locr\WordPress\Plugin\LocrMaps\Admin\Database;
use Locr\WordPress\Plugin\LocrMaps\Api\{Icon, Icons, OnlineMaps, OnlineMapLocation, Settings};
use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\OnlineMap;
use Locr\WordPress\Plugin\LocrMaps\DDD\Repositories\OnlineMaps as OnlineMapsRepository;
use Locr\WordPress\Plugin\LocrMaps\Filters\{Icons as FiltersIcons, Styles as FiltersStyles};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the plugin.
 */
final class Plugin {
	public const LEAFLET_VERSION = '1.9.4';
	public const SLUG            = 'locrmaps';

	/**
	 * This function initializes the plugin.
	 */
	public static function init(): void {
		self::init_textdomain();
		self::init_admin_hooks();
		self::init_public_hooks();
		FiltersIcons::init();
		FiltersStyles::init();

		do_action( 'locrmaps_initialized' );
	}

	/**
	 * This function initializes the admin hooks.
	 */
	private static function init_admin_hooks(): void {
		Admin\SettingsPage::init();
	}

	/**
	 * This function initializes the public hooks.
	 */
	private static function init_public_hooks(): void {
		Frontend\Shortcode::init();
		Frontend\Style::init();
	}

	/**
	 * This function initializes the public hooks.
	 */
	private static function init_textdomain(): void {
		add_action(
			'init',
			function () {
				load_plugin_textdomain(
					domain: self::SLUG . '-textdomain',
					deprecated: false,
					plugin_rel_path: dirname( dirname( plugin_basename( __FILE__ ) ) ) . DIRECTORY_SEPARATOR . 'languages'
				);

				$onlinemaps                   = new OnlineMapsRepository();
				$all_onlinemaps               = $onlinemaps->get_all();
				$onlinemaps_attribute_default = array();
				foreach ( $all_onlinemaps as $onlinemap ) {
					if ( $onlinemap instanceof OnlineMap ) {
						$onlinemaps_attribute_default[ $onlinemap->id->value ] = (string) $onlinemap->name;
					}
				}

				register_block_type(
					dirname( __DIR__, 2 ) . DIRECTORY_SEPARATOR . 'build',
					array(
						'attributes' => array(
							'width'     => array(
								'type'    => 'string',
								'default' => '',
							),
							'height'    => array(
								'type'    => 'string',
								'default' => '',
							),
							'onlinemap' => array(
								'type'    => 'int',
								'default' => 0,
							),
						),
					)
				);

				$languages_directory = dirname( plugin_dir_path( __FILE__ ) ) . DIRECTORY_SEPARATOR . 'languages';
				wp_set_script_translations(
					handle: 'locr-company-locrmaps-wp-plugin-editor-script',
					domain: self::SLUG . '-textdomain',
					path: $languages_directory
				);
				wp_set_script_translations(
					handle: 'locr-company-locrmaps-wp-plugin-view-script',
					domain: self::SLUG . '-textdomain',
					path: $languages_directory
				);
			}
		);
		add_action(
			'rest_api_init',
			function () {
				Icons::register_all_icons_route();
				Icon::register_icon_route();
				OnlineMaps::register_all_onlinemaps_route();
				OnlineMapLocation::register_add_location_route();
				OnlineMapLocation::register_update_location_route();
				Settings::register_all_options_route();
				Settings::register_general_options_route();
				Settings::register_icon_options_route();
			}
		);
		add_filter(
			'load_textdomain_mofile',
			function ( string $mofile, string $domain ) {
				if ( self::SLUG . '-textdomain' === $domain && false !== strpos( $mofile, WP_LANG_DIR . '/plugins/' ) ) {
					$locale = apply_filters(
						hook_name: 'locrmaps_locale',
						value: determine_locale(),
						args: $domain
					);
					if ( ! is_string( $locale ) ) {
						$locale = 'en_US';
					}
					$mofile = WP_PLUGIN_DIR . '/' . dirname( dirname( plugin_basename( __FILE__ ) ) ) . '/languages/' . $domain . '-' . $locale . '.mo';
				}
				return $mofile;
			},
			10,
			2
		);
		add_filter(
			'load_script_translation_file',
			function ( string|false $file, string $handle, string $domain ): string|false {
				if ( false !== $file && ( 'locr-company-locrmaps-wp-plugin-view-script' === $handle || 'locr-company-locrmaps-wp-plugin-editor-script' === $handle ) ) {
					$file = str_replace( WP_LANG_DIR . DIRECTORY_SEPARATOR . 'plugins', plugin_dir_path( __FILE__ ) . 'src' . DIRECTORY_SEPARATOR . 'languages', $file );
				}

				return $file;
			},
			10,
			3
		);
	}

	/**
	 * This function does cleanups if the plugin is being uninstalled.
	 */
	public static function uninstall(): void {
		Database::drop_all_plugin_tables();
	}
}
