<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects;

use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\GeneralOptions\AccessKey;
use Locr\WordPress\Plugin\LocrMaps\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the options.
 *
 * @property-read AccessKey $access_key The access key.
 */
final class GeneralOptions {
	public const OPTION_NAME = Plugin::SLUG . '_general_options';

	/**
	 * The access key.
	 *
	 * @var AccessKey
	 */
	private AccessKey $access_key;

	/**
	 * The constructor.
	 *
	 * @param array<mixed> $shortcode_attributes The shortcode attributes.
	 * @param string       $only_option The only option.
	 */
	public function __construct( array $shortcode_attributes = array(), private string $only_option = '' ) {
		$this->access_key = new AccessKey();

		$this->parse_options();
		$this->parse_attributes( $shortcode_attributes );
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'access_key' => $this->access_key,
			default => throw new \InvalidArgumentException( "Property $name does not exist" ), // phpcs:ignore
		};
	}

	/**
	 * This function parses the attributes.
	 *
	 * @param array<mixed> $shortcode_attributes The shortcode attributes.
	 */
	private function parse_attributes( array $shortcode_attributes = array() ): void {
		if ( ( empty( $this->only_option ) || AccessKey::OPTION_KEY === $this->only_option ) && isset( $shortcode_attributes[ AccessKey::ATTRIBUTE_KEY ] ) && is_string( $shortcode_attributes[ AccessKey::ATTRIBUTE_KEY ] ) ) {
			$this->access_key = new AccessKey( trim( $shortcode_attributes[ AccessKey::ATTRIBUTE_KEY ] ) );
		}
	}

	/**
	 * This function parses the options.
	 */
	private function parse_options(): void {
		$options = get_option( self::OPTION_NAME );
		if ( ! is_array( $options ) ) {
			return;
		}

		if ( ( empty( $this->only_option ) || AccessKey::OPTION_KEY === $this->only_option ) && isset( $options[ AccessKey::OPTION_KEY ] ) && is_string( $options[ AccessKey::OPTION_KEY ] ) ) {
			$this->access_key = new AccessKey( trim( $options[ AccessKey::OPTION_KEY ] ) );
		}
	}

	/**
	 * This function returns the options as an array.
	 *
	 * @return array<mixed>
	 */
	public function to_array(): array {
		return array(
			AccessKey::ATTRIBUTE_KEY => (string) $this->access_key,
		);
	}

	/**
	 * This function returns the options as JSON.
	 */
	public function to_json(): string {
		$json = wp_json_encode( $this->to_array() );
		if ( false === $json ) {
			return '';
		}

		return $json;
	}
}
